#############################################
# Created By: Zack Johnson, June 8, 2020
# Most recently editied on: June 8, 2020

# Task: Use RStan to make a model that predicts the index of 3 binary variables (subed_IDC, subtax_IDC, and subpolice_IDC)
# as related to the subnational policy authority that a given country has.
#############################################

# Clear lists
rm(list=ls())

# Set working directory
setwd("/Volumes/GoogleDrive/Shared drives/SPEC Powersharing S20/data")

# Read in the libraries
library(readr)
library(tidyverse)
library(rstan)

# Load in data
load("MERGED_IDC_Controls_MB_05222020.RData")

# Isolate just the variables we're interested in for this model, rename the vars to simpler names, and make an additive index
model_data <-
  idc_controls %>%
  select(country:gwabbrev,
         subtax_IDC:subpolice_IDC) %>%
  mutate(subtax = subtax_IDC, subed = subed_IDC, subpolice = subpolice_IDC) %>%
  select(country:gwabbrev, subtax:subpolice) %>%
  mutate(additive = subtax + subed + subpolice)

# Number of observations
n <- nrow(model_data)

# define j as the number of items
j <- 3

# make a matrix of the item response
y <- cbind(model_data$subtax, model_data$subed, model_data$subpolice)

# make a column vector of the item response with missing values excluded
y_missing <- which(!is.na(y))
y <- y[y_missing]

# scalar for the total number of subjects by observed items
n_j <- length(y)

# make a column vector of the item numbers with missing values excluded
item <- matrix(c(1:j),ncol=j,nrow=n, byrow=T)
item <- c(item)
item <- item[y_missing]


# make a column vector of the subject ids with missing values excluded
id <- matrix(1:n,ncol=j,nrow=n, byrow=F)
id <- c(id)
id <- id[y_missing]


# create data list to pass to STAN
data <- list(y=y, j=j, n=n, n_j=n_j, item=item, id=id)

# set time start variable
time1 <- Sys.time()
print(Sys.time() - time1)


# -------------------------------------------------- #
# define STAN model
# -------------------------------------------------- #
model <- "
data {
  int<lower=0> n; // number of subject observations
  int<lower=0> j; // number of items
  int<lower=0> n_j; // number of non-missing subject-item observations
    
  int<lower=0> item[n_j]; // vector of length n_k that distingishes the value of the k item/model parameters
  int<lower=0> id[n_j]; // vector of length n_k that distingishes the value of the n latent variable parameters
  
  //indicators
  int<lower=0, upper=1> y[n_j];
  
  //int<lower=0, upper=1> subtax[n];
  //int<lower=0, upper=1> subpolice[n];
                                      // Converted non-missing values from the 3 indicators into one long vector so 
                                      // I only need to declare one vector for the indicators
}

// The parameters accepted by the model. Our model
// accepts two parameters 'mu' and 'sigma'.
parameters {
  vector[j] alpha;
  real<lower=0> beta[j];
  vector[n] theta;
}
transformed parameters { // deterministic transformation of parameters to 
    vector[n_j] alpha_star; // column vector of alpha parameters
    real<lower=0> beta_star[n_j]; // column vector of beta paramets
    vector[n_j] theta_star; // column vector of latent variable parameters
    //vector[n_j] xb; // column vector of linear combination of parameter column vectors
    
    for(i in 1:n_j){
        // loop through parameters vectors to generate column vectors
        alpha_star[i] = alpha[item[i]];
        beta_star[i] = beta[item[i]];
        theta_star[i] = theta[id[i]];
    }
    
}

// The model to be estimated. We model the output
// 'y' to be normally distributed with mean 'mu'
// and standard deviation 'sigma'.
model {
  theta ~ normal(0,1); //priors on latent variables
  
  alpha ~ normal(0,4); //priors (these are variances not precision)
  beta ~ normal(0,1);
  
  //Items
  y ~ bernoulli_logit(alpha_star + beta_star * theta_star)
  
}
"

fit <- stan(model_code = model, data = data, iter = 1000, chains = 4)

fit